SET NOCOUNT ON;

INSERT INTO msn.OptionContracts
    ( [Symbol]
    , [ExpDate]
    , [Type]
    , [Strike]
    , [ContractName]
    , [PairContractName]
    )
SELECT
    DISTINCT
    t.[Symbol]
    , t.[ExpDate]
    , t.[Type]
    , t.[Strike]
    , t.[ContractName]
    , CASE
        WHEN t.[Type] = 'C' THEN REPLACE(t.[ContractName], 'C0', 'P0')
        ELSE REPLACE(t.[ContractName], 'P0', 'C0')
        END AS [PairContractName]
FROM
    (
    SELECT
        [Symbol] AS [Symbol]
        , [ExpDate] AS [ExpDate]
        , SUBSTRING([Type], 1, 1) AS [Type]
        , [Strike] AS [Strike]
        , [ContractName] AS [ContractName]
    FROM
        OPENROWSET(
            BULK 'C:\MarketData\csv\MsnMoneyOptions.csv',
            FORMATFILE = 'C:\MarketData\bin\MsnMoneyOptions.fmt',
            CODEPAGE = '1252',
            FIRSTROW = 2) t
    ) t
    LEFT OUTER JOIN msn.OptionContracts d ON d.ContractName = t.ContractName COLLATE SQL_Latin1_General_CP1_CI_AS
WHERE
    d.ContractName IS NULL
    AND t.Strike IS NOT NULL
    AND t.ExpDate < DATEADD(year, 5, GETDATE())
    AND LEFT(LEFT(t.ContractName, LEN(t.Symbol)), 2) = LEFT(t.Symbol, 2)
GO

INSERT INTO msn.Options
    ( OptionContractID
    , [LoadDate]
    , [LoadTime]
    , [MarketDate]
    , [MarketTime]
    , [Last]
    , [Bid]
    , [Ask]
    , [Change]
    , [ChangePercent]
    , [Volume]
    , [OpenInterest]
    )
SELECT
    c.ID 
    , t.[LoadDate]
    , t.[LoadTime]
    , t.[MarketDate]
    , t.[MarketTime]
    , t.[Last]
    , t.[Bid]
    , t.[Ask]
    , t.[Change]
    , t.[ChangePercent]
    , t.[Volume]
    , t.[OpenInterest]
FROM
    (
    SELECT
        [LoadDate] AS [LoadDate]
        , [LoadTime] AS [LoadTime]
        , [MarketDate] AS [MarketDate]
        , [MarketTime] AS [MarketTime]
        , [Symbol] AS [Symbol]
        , [ExpDate] AS [ExpDate]
        , SUBSTRING([Type], 1, 1) AS [Type]
        , [Strike] AS [Strike]
        , [ContractName] AS [ContractName]
        , [Last] AS [Last]
        , [Bid] AS [Bid]
        , [Ask] AS [Ask]
        , [Change] AS [Change]
        , CAST(REPLACE([ChangeInPercent], '%', '') AS float) / 100  AS [ChangeInPercent]
        , [Volume] AS [Volume]
        , [OpenInterest] AS [OpenInterest]
        , ROW_NUMBER() OVER (PARTITION BY [LoadDate], [LoadTime], [ContractName] ORDER BY [LoadTime]) AS [Row]
    FROM
        OPENROWSET(
            BULK 'C:\MarketData\csv\MsnMoneyOptions.csv',
            FORMATFILE = 'C:\MarketData\bin\MsnMoneyOptions.fmt',
            CODEPAGE = '1252',
            FIRSTROW = 2) t
    ) t
    INNER JOIN msn.OptionContracts c ON c.ContractName = t.ContractName COLLATE SQL_Latin1_General_CP1_CI_AS
    LEFT OUTER JOIN msn.Options d
        ON d.OptionContractID = c.ID AND d.LoadDate = t.LoadDate AND d.LoadTime = t.LoadTime
WHERE
    d.OptionContractID IS NULL
    AND t.Strike IS NOT NULL
    AND t.ExpDate < DATEADD(year, 5, GETDATE())
    AND LEFT(LEFT(t.ContractName, LEN(t.Symbol)), 2) = LEFT(t.Symbol, 2)
    AND t.[Row] = 1
